@@ -1,13 +1,11 @@
-
 package Shell::Perl::Dumper;
 
-use 5;
 use strict;
 use warnings;
 
 # $Id$
 
-our $VERSION = '0.0017';
+our $VERSION = '0.0024';
 
 use base qw(Class::Accessor); # to get a new() for free
 
@@ -36,39 +34,29 @@ our @ISA = qw(Shell::Perl::Dumper); # to get a new() for free
 
 # XXX make a Data::Dump object an instance variable
 
-sub require_data_dump {
-    require Data::Dump;
+sub _dump_code_filter {
+    my ($ctx, $object_ref) = @_;
+    return undef unless $ctx->is_code;
 
-    # XXX hack Data::Dump as it does not dump code
-    my $_dump = \&Data::Dump::_dump; # original
-
-    no warnings 'redefine';
-    *Data::Dump::_dump = sub {
-        my $val = $_[0];
-        my $out = $_dump->(@_);
-        if ( $out eq 'sub { "???" }' ) {
-            require B::Deparse;
-            return 'sub ' . (B::Deparse->new)->coderef2text($val);
-        } else {
-            return $out
-        }
-    }
+    require B::Deparse;
+    my $code = 'sub ' . (B::Deparse->new)->coderef2text($object_ref);
+    return { dump => $code };
 }
 
 sub is_available {
-    return eval { require_data_dump; 1 };
+    return eval { require Data::Dump::Filtered; 1 };
 }
 
 sub dump_scalar {
     shift;
-    require Data::Dump; 
-    return Data::Dump::dump(shift) . "\n";
+    require Data::Dump::Filtered;
+    return Data::Dump::Filtered::dump_filtered(shift, \&_dump_code_filter) . "\n";
 }
 
 sub dump_list {
     shift;
-    require Data::Dump; 
-    return Data::Dump::dump(@_) . "\n";
+    require Data::Dump::Filtered;
+    return Data::Dump::Filtered::dump_filtered(@_, \&_dump_code_filter) . "\n";
 }
 
 package Shell::Perl::Data::Dumper;
@@ -84,7 +72,7 @@ sub is_available {
 
 sub dump_scalar {
     shift;
-    require Data::Dumper; 
+    require Data::Dumper;
     local $Data::Dumper::Deparse = 1;
     return Data::Dumper->Dump([shift], [qw($var)]);
 }
@@ -92,7 +80,7 @@ sub dump_scalar {
 sub dump_list {
     #goto &dump_scalar if @_==2; # fallback to dump_scalar if only one
     shift;
-    require Data::Dumper; 
+    require Data::Dumper;
     local $Data::Dumper::Deparse = 1;
     return Data::Dumper->Dump([[@_]], [qw(*var)]);
 }
@@ -148,14 +136,14 @@ sub is_available {
 
 sub dump_scalar {
     shift;
-    require Data::Dump::Streamer; 
+    require Data::Dump::Streamer;
     return Data::Dump::Streamer::Dump(shift)->Names('$var')->Out;
 }
 
 sub dump_list {
     #goto &dump_scalar if @_==2; # fallback to dump_scalar if only one
     shift;
-    require Data::Dump::Streamer; 
+    require Data::Dump::Streamer;
     return Data::Dump::Streamer::Dump([@_])->Names('*var')->Out;
 }
 
@@ -166,6 +154,10 @@ sub dump_list {
 
 __END__
 
+=pod
+
+=encoding utf-8
+
 =head1 NAME
 
 Shell::Perl::Dumper - Dumpers for Shell::Perl
@@ -187,13 +179,13 @@ for you to pretty-print these answers just the way you want.
 By default, C<pirl> will try to convert the results
 via C<Data::Dump>. That means the output will be Perl
 code that may be run to get the data structure again.
-Alternatively, the shell may use C<Data::Dumper> 
+Alternatively, the shell may use C<Data::Dumper>
 or C<Data::Dump::Streamer>
 with almost the same result with respect to the
 representation as Perl code. (But the output of the
 modules differ enough for sufficiently complex data.)
 
-Other options are to set the output to produce YAML 
+Other options are to set the output to produce YAML
 or a plain simple-minded solution which basically
 turns the result to string via simple interpolation.
 
@@ -248,7 +240,7 @@ an installation problem.
 This is the algorithm used by L<Shell::Perl> XXX
 XXX XXX
 
-    1. 
+    1.
 
 =back
 
@@ -359,7 +351,7 @@ Examples of its output:
     --- a
 
 When loading, C<YAML::Syck> is preferred to C<YAML>. If it
-is not avaiable, the C<YAML> module is the second option.
+is not available, the C<YAML> module is the second option.
 
 =head2 Data::Dump::Streamer
 
@@ -391,22 +383,11 @@ Examples of its output:
 
 =head1 SEE ALSO
 
-    Shell::Perl
-
-=head1 BUGS
-
-Please report bugs via CPAN RT L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Shell-Perl>
-or L<mailto://bugs-Shell-Perl@rt.cpan.org>.
-
-=head1 AUTHORS
-
-Adriano R. Ferreira, E<lt>ferreira@cpan.orgE<gt>
-
-Caio Marcelo, E<lt>cmarcelo@gmail.comE<gt>
+L<Shell::Perl>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2007 by Adriano R. Ferreira
+Copyright (C) 2007–2017 by Adriano R. Ferreira
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
